/**
 * Helper functions for calculating limits for the FlashIAP block device
 * */

#pragma once

#include <Arduino.h>
#include <FlashIAP.h>
#include <FlashIAPBlockDevice.h>

using namespace mbed;

// An helper struct for FlashIAP limits
struct FlashIAPLimits {
    size_t flash_size;
    uint32_t start_address;
    uint32_t available_size;
};

// Get the actual start address and available size for the FlashIAP Block Device
// considering the space already occupied by the sketch.
FlashIAPLimits getFlashIAPLimits()
{
    // Alignment lambdas
    auto align_down = [](uint64_t val, uint64_t size) { return (((val) / size)) * size; };
    auto align_up = [](uint32_t val, uint32_t size) { return (((val - 1) / size) + 1) * size; };

    size_t flash_size;
    uint32_t flash_start_address;
    uint32_t start_address;
    FlashIAP flash;

    auto result = flash.init();
    if (result != 0)
        return { };

    // Find the start of first sector after text area
    int sector_size = flash.get_sector_size(FLASHIAP_APP_ROM_END_ADDR);
    start_address = align_up(FLASHIAP_APP_ROM_END_ADDR, sector_size);
    flash_start_address = flash.get_flash_start();
    flash_size = flash.get_flash_size();

    result = flash.deinit();

    int available_size = flash_start_address + flash_size - start_address;
    if (available_size % (sector_size * 2)) {
        available_size = align_down(available_size, sector_size * 2);
    }

    return { flash_size, start_address, available_size };
}
